clear;

%%
c = 343.1;
f0 = 1e3;
xs = [0.4 2.5 0.];

[pot_bb, pot_path] = read_epspath('krumpli2.eps');
siz = norm(diff(pot_bb));
rad_int = meshpath(pot_path, siz/300);
rad_int = translate_mesh(scale_mesh(rad_int, 1/siz*5.25), [1.5 .5]);
rad_ext = rad_int;
rad_ext.Elements = flip_elements(rad_ext.Elements);


Lx = 6.25;
Ly = 4;
field = create_slab([Lx, Ly], round([Lx Ly]*60));

%// extract the mesh description matrices
[f_nodes, f_elem] = extract_core_mesh(field);
%// convert slabs to 2D volume elements
f_elem(:,1) = 22404;

%%
%// call C++ code at wave number k = 4
%k = .7 * min(min(mesh_kmax(rad_int)), min(mesh_kmax(field)))*1.5;
k = 2*pi*f0/c;
%% interior
[r_nodes, r_elem] = extract_core_mesh(rad_int);
[Ls_int, Ms_int, Lf_int, Mf_int] = helmholtz_bem_2d(r_nodes, r_elem, f_nodes, f_elem, k);

%% exterior
[r_nodes, r_elem] = extract_core_mesh(rad_ext);
[Ls_ext, Ms_ext, Lf_ext, Mf_ext] = helmholtz_bem_2d(r_nodes, r_elem, f_nodes, f_elem, k);

%%

%% analytical incident on the field
f_cent = centnorm(field);
P_ana = incident('line', xs, f_cent, [], k);


%% solve interior
r_cent = centnorm(rad_int);
ps_ana = incident('line', xs, r_cent, [], k);
qs_int = Ls_int \ ((Ms_int - .5 * eye(size(Ms_int))) * ps_ana);
%pf_int = Mf_int * ps_ana - Lf_int * qs_ana;

% figure;
% subplot(2,1,1);
% plot_mesh(field, real(pf_int));
% plot_mesh(rad_int);
% caxis([-1 1] * 1e-1);
% shading flat;
% subplot(2,1,2);
% plot_mesh(field, real(pf_ana));
% plot_mesh(rad_int);
% caxis([-1 1] * 1e-1);
% shading flat;

%% solve exterior
r_cent = centnorm(rad_ext);
ps_ana = incident('line', xs, r_cent, [], k);
qs_ext = Ls_ext \ ((Ms_ext - .5 * eye(size(Ms_ext))) * ps_ana);
pf_ext = Mf_ext * ps_ana - Lf_ext * qs_ext;


% figure;
% subplot(2,2,1);
% plot_mesh(field, real(pf_ext));
% plot_mesh(rad_int);
% caxis([-1 1] * 1e-1);
% shading flat;
% subplot(2,2,2);
% plot_mesh(field, real(pf_ana));
% plot_mesh(rad_int);
% caxis([-1 1] * 1e-1);
% shading flat;
% subplot(2,2,3);
% plot_mesh(field, real(pf_ana - pf_ext));
% plot_mesh(rad_int);
% caxis([-1 1] * 1e-1);
% shading flat;


%% combined with inward normal
qs = -qs_ext - qs_int;

[C,n] = centnorm(rad_int);
vx = bsxfun(@minus,C,xs);
vn = sum(vx.*n,2);
w = sum(vx.*n,2);
win = (w<=0);
P_simple_source = Lf_int * qs;           
P_KHA = -2*Lf_int * (win.*qs_ext); % KH-approximation
%% Simple source figure
ftsize = 14;
f = figure('Units','points','Position',[200,0,700,500]);

subplot(2,2,1);
plot_mesh(field, real(P_ana));
set(gca, 'Units','normalized','Position',[ 0.05 0.54 0.42 .45 ]);
x = rad_int.Nodes(1:10:end,2);
y = rad_int.Nodes(1:10:end,3);
z = rad_int.Nodes(1:10:end,4);
line(x, y, z, 'Color', 'black','LineStyle','--');
caxis([-1 1] * 0.5e-1);
shading flat;
axis equal tight
set(gca,'box','on')
xlabel('$x \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
ylabel('$z \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
set(gca,'FontName','Times New Roman');
allAxesInFigure = findall(f,'type','axes');
b = get(gca,'XTickLabel');

subplot(2,2,2);
plot_mesh(field, real( P_simple_source ));
set(gca, 'Units','normalized','Position',[ 0.57 0.54 0.42 .45 ]);
x = rad_int.Nodes(:,2);
y = rad_int.Nodes(:,3);
z = rad_int.Nodes(:,4);
line(x, y, z, 'Color', 'black','LineStyle','-');
caxis([-1 1] * 0.5e-1);
shading flat;
axis equal tight
set(gca,'box','on')
xlabel('$x \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
ylabel('$z \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
set(gca,'FontName','Times New Roman');
allAxesInFigure = findall(f,'type','axes');
b = get(gca,'XTickLabel');

subplot(2,2,3:4);
plot_mesh(field, real(P_ana - P_simple_source ));
set(gca, 'Units','normalized','Position',[ 0.33 0.03 0.42 .45 ]);
line(x, y, z, 'Color', 'black','LineStyle','-');
caxis([-1 1] * 0.5e-1);
shading flat;
axis equal tight
set(gca,'box','on')
xlabel('$x \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
ylabel('$z \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
set(gca,'FontName','Times New Roman');
allAxesInFigure = findall(f,'type','axes');
b = get(gca,'XTickLabel');


set(gcf,'PaperPositionMode','auto');
%print -dpng simple_source_formulation_2 -r300
%
%% Kirchhoff approximation figure
ftsize = 14;
f = figure('Units','points','Position',[200,200,700,250]);

subplot(1,2,1);
plot_mesh(field, real( P_KHA ));
set(gca, 'Units','normalized','Position',[ 0.05 0.1 0.42 .9 ]);
x = rad_int.Nodes(:,2);
y = rad_int.Nodes(:,3);
z = rad_int.Nodes(:,4);
line(x, y, z, 'Color', 'black','LineStyle',':','LineWidth',2);
line(rad_int.Nodes(win,2), rad_int.Nodes(win,3), rad_int.Nodes(win,4), 'Color', 'black','LineStyle','-','LineWidth',3);
caxis([-1 1] * 0.5e-1);
shading flat;
axis equal tight
set(gca,'box','on')
xlabel('$x \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
ylabel('$z \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
set(gca,'FontName','Times New Roman');
allAxesInFigure = findall(f,'type','axes');
b = get(gca,'XTickLabel');

subplot(1,2,2);
plot_mesh(field, real(P_ana - P_KHA ));
set(gca, 'Units','normalized','Position',[ 0.57 0.1 0.42 .9 ]);
line(x, y, z, 'Color', 'black','LineStyle','-');
caxis([-1 1] * 0.5e-1);
shading flat;
axis equal tight
set(gca,'box','on')
xlabel('$x \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
ylabel('$z \rightarrow \mathrm{[m]}$','Interpreter','latex','FontSize',ftsize)
set(gca,'FontName','Times New Roman');
allAxesInFigure = findall(f,'type','axes');
b = get(gca,'XTickLabel');


set(gcf,'PaperPositionMode','auto');
%print -dpng KH_approx -r300