	PROGRAM AEBEM3TESTSOL
		use COMPATIBILITY, only: READ_OFF_MESH, CALC_TRIA_CENTERS
		use ASSEMBLY, only: AEBEM3ASM
		use SOLUTION, only: CLINSL
		implicit none
C Print out the list of parameters
		INTEGER :: i
		INTEGER :: j
		
		real*8, allocatable :: nodes(:,:)
		integer, allocatable :: elements(:,:)
		real*8, allocatable :: centers(:,:)
		real*8, allocatable :: normals(:,:)
		
		real*8 :: K

		CHARACTER(len=256) :: tempstr
		CHARACTER(len=256) :: meshfile
		CHARACTER(len=256) :: line

		COMPLEX*16, ALLOCATABLE :: LMAT(:,:)
		COMPLEX*16, ALLOCATABLE :: MMAT(:,:)
		COMPLEX*16, ALLOCATABLE :: Q(:)
		COMPLEX*16, ALLOCATABLE :: RHS(:)
		COMPLEX*16, ALLOCATABLE :: SOL(:)
		REAL*8, ALLOCATABLE :: ERR(:)
C Mean error
		REAL*8 MEANERR
C The analytical solution
		COMPLEX*16 :: ANAL

		real :: t_elaps, t(2)

C Check arguments
		IF (iargc() < 2) THEN
			write(*,*) "USAGE: AEBEM3TEST meshfile k"
			STOP 1
		END IF
C Obtain meshfile from parameter
		CALL getarg(1, meshfile)
C Obtain wavenumber from parameter
		CALL getarg(2, tempstr)
		read(tempstr,*) K
C Write k to file
		open(unit=10,file="k.out",action="write",status="replace")
		write(10,*) K
		close(10)
C Read the mesh
		CALL etime(t, t_elaps)
		write(*,*) "Reading mesh file: ", meshfile
		CALL READ_OFF_MESH(meshfile, nodes, elements)
		CALL etime(t, t_elaps)
		write(*,*) "Elapsed time: ", t_elaps
		write(*,*) "Mesh nodes: ", SIZE(nodes,1)
		write(*,*) "Mesh elements: ", SIZE(elements,1)
C -------------------------------------
C Calculate element normals and centers
C -------------------------------------
		CALL etime(t, t_elaps)
		write(*,*) "Calculating elem centers and normals ..."
		CALL CALC_TRIA_CENTERS(nodes, elements, centers, normals)
		CALL etime(t, t_elaps)
		write(*,*) "Elapsed time: ", t_elaps
		CALL etime(t, t_elaps)
		write (*,*) "Assembling matrices at k = ", K, "..."
		CALL AEBEM3ASM(K,nodes,elements,centers,normals,
     *                 LMAT, MMAT)
		CALL etime(t, t_elaps)
		write(*,*) "Elapsed time: ", t_elaps
C ----------------
C Assemble the RHS
C ----------------
		CALL etime(t, t_elaps)
		write(*,*) "Assembling RHS ..."
C Preallocate
		allocate(Q(SIZE(elements,1)))
		allocate(RHS(SIZE(elements,1)))
C Fill excitation vector
		DO i=1,SIZE(elements,1)
			Q(i) = CMPLX(1.0D0,0.0D0)
		END DO
C Fill RHS vector
		DO i=1,SIZE(elements,1)
			RHS(i) = CMPLX(0.0D0,0.0D0)
			DO j=1,SIZE(elements,1)
				RHS(i) = RHS(i) + LMAT(i,j)*Q(j)
			END DO
		END DO
C Free the L matrix to save RAM
		deallocate(LMAT)
		CALL etime(t, t_elaps)
		write(*,*) "Elapsed time: ", t_elaps
C --------
C Solution
C --------
		CALL etime(t, t_elaps)
		write(*,*) "Solution ..."
C Call solution function
		CALL CLINSL(MMAT,RHS,SOL)
		CALL etime(t, t_elaps)
		write(*,*) "Elapsed time: ", t_elaps
C -------------------
C Calculate the error
C -------------------
		CALL etime(t, t_elaps)
		write(*,*) "Calculating L2 error ..."
C Preallocate
		allocate(ERR(SIZE(elements,1)))
		ANAL = CMPLX(-1.0D0,0.0D0)/(CMPLX(1.0D0,-K))
		MEANERR = 0.0D0
		DO i=1,SIZE(elements,1)
			ERR(i) = ABS(SOL(i)-ANAL)
			MEANERR = MEANERR+ERR(i)
		END DO
		MEANERR = MEANERR/SIZE(elements,1)
		CALL etime(t, t_elaps)
		write(*,*) "Elapsed time: ", t_elaps
		write(*,*) "Mean L2 error is: ", MEANERR
C Deallocate variables		
		deallocate(nodes)
		deallocate(elements)
		deallocate(centers)
		deallocate(normals)
		deallocate(MMAT)
		deallocate(RHS)
		deallocate(SOL)
		deallocate(ERR)
	END PROGRAM
