/** \file specialise.hpp 
* \details Tutorial demonstrating how to specialise a CRTP base class in NiHu
* \author Peter Fiala and Peter Rucz
* \ingroup tutorials
*/

//! [Internal]
// forward declaration of entity traits
template <class CustomEntity>
entity_traits;

// the base class
template <class CustomEntity>
class entity_base
{
public:
	// get information on the custom class from its traits class
	typedef typename entity_traits<CustomEntity>::scalar_t scalar_t;
	
	// define a new type based on scalar_t
	typedef scalar_t *scalar_ptr;
	
	// a member function interface
	scalar_t do_it(scalar_ptr input)
	{
		return static_cast<CustomEntity &>(*this).do_it(input);
	}
	
	// a static member function interface
	static scalar_t eval(void)
	{
		return CustomEntity::eval();
	}
	
	// an abstract algorithm
	scalar_t abstract_algorithm(void)
	{
		scalar_t a = eval();
		return do_it(&a);
	}
};
//! [Internal]


//! [Declare]
// forward declaration to be able to use as template parameter
class custom_entity;
//! [Declare]

//! [Traits]
// specializing entity_traits
template <>
struct entity_traits<custom_entitiy>
{
	typedef double scalar_t;
};
//! [Traits]

//! [Class]
class custom_entity : public entity_base<custom_entity>
{
public:
	scalar_t do_it(scalar_ptr input)
	{
		return *input;
	}
	
	static scalar_t eval(void)
	{
		return 1.0;
	}
};
//! [Class]


//! [Custom template]
// forward declaration to be able to use as template parameter
template <class T>
class custom_entity;

template <class T>
struct entity_traits<custom_entitiy<T> >
{
	typedef T scalar_t;
};


template <class T>
class custom_entity : public entity_base<custom_entity<T> >
{
public:
	typedef entity_base<custom_entity<T> > base_t;
	typedef typename base_t::scalar_t scalar_t;
	typedef typename base_t::scalar_ptr scalar_ptr;
	
	scalar_t do_it(base_t::scalar_ptr input)
	{
		return *input;
	}
	
	static scalar_t eval(void)
	{
		return 1.0;
	}
};
//! [Custom template]

